@@ -2,19 +2,21 @@
 
 import os
 import sys
+import glob
 import socket
 import argparse
 import traceback
 import _pickle as pickle
+from prettytable import PrettyTable
 
 from pimdm import Main
 from pimdm.tree import pim_globals
 from pimdm.daemon.Daemon import Daemon
 
-VERSION = "1.1.1.2"
+VERSION = "1.1.1.4"
 
 
-def client_socket(data_to_send):
+def client_socket(data_to_send, print_output=True):
     # Create a UDS socket
     sock = socket.socket(socket.AF_UNIX, socket.SOCK_STREAM)
 
@@ -26,7 +28,10 @@ def client_socket(data_to_send):
         sock.sendall(pickle.dumps(data_to_send))
         data_rcv = sock.recv(1024 * 256)
         if data_rcv:
-            print(pickle.loads(data_rcv))
+            if print_output:
+                print(pickle.loads(data_rcv))
+            else:
+                return pickle.loads(data_rcv)
     except socket.error:
         pass
     finally:
@@ -92,6 +97,8 @@ class MyDaemon(Daemon):
                 elif 'remove_interface_mld' in args and args.remove_interface_mld:
                     Main.remove_interface(args.remove_interface_mld[0], membership=True, ipv4=False, ipv6=True)
                     connection.shutdown(socket.SHUT_RDWR)
+                elif 'list_instances' in args and args.list_instances:
+                    connection.sendall(pickle.dumps(Main.list_instances()))
                 elif 'stop' in args and args.stop:
                     Main.stop()
                     connection.shutdown(socket.SHUT_RDWR)
@@ -121,6 +128,8 @@ def main():
                                                                                            "Use -4 or -6 to specify IPv4 or IPv6 PIM neighbors.")
     group.add_argument("-ls", "--list_state", action="store_true", default=False, help="List IGMP/MLD and PIM-DM state machines."
                                                                                        " Use -4 or -6 to specify IPv4 or IPv6 state respectively.")
+    group.add_argument("-instances", "--list_instances", action="store_true", default=False,
+                       help="List running PIM-DM daemon processes.")
     group.add_argument("-mr", "--multicast_routes", action="store_true", default=False, help="List Multicast Routing table. "
                                                                                              "Use -4 or -6 to specify IPv4 or IPv6 multicast routing table.")
     group.add_argument("-ai", "--add_interface", nargs=1, metavar='INTERFACE_NAME', help="Add PIM interface. "
@@ -139,7 +148,7 @@ def main():
     group_ipversion = parser.add_mutually_exclusive_group(required=False)
     group_ipversion.add_argument("-4", "--ipv4", action="store_true", default=False, help="Setting for IPv4")
     group_ipversion.add_argument("-6", "--ipv6", action="store_true", default=False, help="Setting for IPv6")
-    group_vrf = parser.add_mutually_exclusive_group(required=False)
+    group_vrf = parser.add_argument_group()
     group_vrf.add_argument("-mvrf", "--multicast_vrf", nargs=1, default=[pim_globals.MULTICAST_TABLE_ID],
                            metavar='MULTICAST_VRF_NUMBER', type=int,
                            help="Define multicast table id. This can be used on -start to explicitly start the daemon"
@@ -157,6 +166,21 @@ def main():
     if os.geteuid() != 0:
         sys.exit('PIM-DM must be run as root!')
 
+    if args.list_instances:
+        pid_files = glob.glob("/tmp/Daemon-pim*.pid")
+        t = PrettyTable(['Instance PID', 'Multicast VRF', 'Unicast VRF'])
+
+        for pid_file in pid_files:
+            d = MyDaemon(pid_file)
+            pim_globals.MULTICAST_TABLE_ID = pid_file[15:-4]
+            if not d.is_running():
+                continue
+
+            t_new = client_socket(args, print_output=False)
+            t.add_row(t_new.split("|"))
+        print(t)
+        return
+
     pim_globals.MULTICAST_TABLE_ID = args.multicast_vrf[0]
     pim_globals.UNICAST_TABLE_ID = args.unicast_vrf[0]
 
